const path = require('path')
const fs = require('fs-extra')

const {
  NODE_MODULES_PATTERN,
  CHANGELOG_PATTERN
} = require('../constants')

const readPackageJson = folder => {
  const packageJsonFile = path.join(folder, 'package.json')

  const configData = fs.readFileSync(packageJsonFile)
  return JSON.parse(configData)
}

const writePackageJson = (folder, config) => {
  const packageJsonFile = path.join(folder, 'package.json')
  const modifiedConfigData = JSON.stringify(config, null, 2).concat('\n')

  fs.writeFileSync(packageJsonFile, modifiedConfigData)
}

const copyTemplatePackageJsonFile = async folder => {
  const packageJsonFile = path.join(folder, 'package.json')
  const packageSkeletonJsonFile = path.join(folder, 'package.json.skeleton')

  if (fs.existsSync(packageSkeletonJsonFile)) {
    await fs.remove(packageJsonFile)
    await fs.rename(packageSkeletonJsonFile, packageJsonFile)
  }
}

const copyTemplateGitIgnoreFile = async folder => {
  const gitIgnoreFile = path.join(folder, '.gitignore')
  const gitIgnoreSkeletonFile = path.join(folder, '.gitignore.skeleton')

  if (fs.existsSync(gitIgnoreSkeletonFile)) {
    await fs.rename(gitIgnoreSkeletonFile, gitIgnoreFile)
  }
}

const copyTemplateEslintrc = async folder => {
  const gitEslintrcFile = path.join(folder, '.eslintrc')
  const gitEslintrcSkeletonFile = path.join(folder, '.eslintrc.skeleton')

  if (fs.existsSync(gitEslintrcSkeletonFile)) {
    await fs.rename(gitEslintrcSkeletonFile, gitEslintrcFile)
  }
}

const copyNpmrc = async folder => {
  const npmrcFile = path.join(folder, '.npmrc')
  const npmrcFileSkeletonFile = path.join(folder, '.npmrc.skeleton')

  if (fs.existsSync(npmrcFileSkeletonFile)) {
    await fs.rename(npmrcFileSkeletonFile, npmrcFile)
  }
}

const copyAllFiles = async (sourceDir, destDir) => {
  const isSkeletonFile = src =>
    !src.match(NODE_MODULES_PATTERN) && !src.match(CHANGELOG_PATTERN)

  await fs.copy(sourceDir, destDir, { filter: isSkeletonFile })
}

module.exports = {
  readPackageJson,
  writePackageJson,
  copyTemplatePackageJsonFile,
  copyTemplateGitIgnoreFile,
  copyTemplateEslintrc,
  copyNpmrc,
  copyAllFiles
}
